package func2graphrenewed;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.jface.window.DefaultToolTip;
import org.eclipse.jface.window.ToolTip;

public class GraphCanvas extends Canvas {
	/**********************************************************
	 *                       VARIABLES                        *
	 **********************************************************/
	private Node root;
	private Tree aTree;
	private double ratioX, ratioY;
	private int maxLevel,offsetY, clientWidth, clientHeight, tooltipCount;
	private String name;
	private HashMap<Integer, int[]> coordinates;
	private GraphCanvas thisCanvas = this;
	private ArrayList<ToolTip> tip;
	
	/**********************************************************
	 *                     Constructor                        *
	 **********************************************************/
	public GraphCanvas(final Composite parent) {
		this(parent, SWT.NULL);
	}

	public GraphCanvas(Composite parent, int style) {
		super( parent, SWT.NONE);
		setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		offsetY=100;
		clientWidth=getClientArea().width;
		clientHeight=getClientArea().height;
		coordinates = new HashMap<>();
		tip = new ArrayList<>();
		
		addControlListener(new ControlAdapter() { /* resize listener. */
			public void controlResized(ControlEvent event) {
				
				if(root!=null ){//&& getClientArea().width>=300
					clientWidth=getClientArea().width;
					clientHeight=getClientArea().height;
					ratioX =  (double)(clientWidth-2)/root.getSize();
					ratioY = (double)(clientHeight)/(maxLevel + offsetY);
					redraw();
				}
			}
		});
		addPaintListener(new PaintListener() {  /*paint listener. */
			public void paintControl(final PaintEvent event) {
				
				paint(event.gc, root, 0);//event.gc
			}
		});
		
		addMouseListener(new MouseListener() {
			
			@Override
			public void mouseUp(MouseEvent e) {
			}
			
			@Override
			public void mouseDown(MouseEvent e) {
				if((e.stateMask & SWT.CTRL)!=0){
					setTooltip(e.x, e.y, tooltipCount++); //Doesn't do anything, can't show more than one tooltip at at time
				}
				else{
					if(!setTooltip(e.x, e.y,0)){
						for(ToolTip t : tip)
							t.hide();
						tip.clear();
						tooltipCount=0;
					}
					else
						tooltipCount=1;
				}
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
		addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				  if( e.keyCode == 'a' && ( e.stateMask &  SWT.CTRL ) != 0 )
					writeContentToFile("f2gImage.jpg");
			}
		});
		
	}
	
	/**********************************************************
	 *                      FUNCTIONS                         *
	 **********************************************************/
	
/* ******************************************************** */
	public void firstDraw(Tree root){
/* ******************************************************** */
//		if(getClientArea().width<300){
//			clientWidth=300;
//		}
		clientWidth=getClientArea().width;
		clientHeight=getClientArea().height;
		this.root=root.getRoot();
		aTree = root;
		maxLevel = root.getMaxLevel();
		name = root.getName();
		ratioX = (double)(clientWidth-2)/this.root.getSize();
		ratioY = (double)(clientHeight)/(maxLevel + offsetY);
		redraw();
	}
	
/* ******************************************************** */
	public void paint(GC gc,Node root, int flag){
/* ******************************************************** */
		if(root==null || ratioX == 0)
			return;
		
		if(flag==0){
			flag=1;
			
			clearCanvas(gc);
			drawLegend(gc,(int)ratioX);
			gc.setForeground(new Color(null, 0,0,0));
			
			paintMainNode(gc, root);
		}
		
		if(!root.hasChild())
			return;
		
		Node currentChild= null;
		while(root.getNextChild(currentChild) != null){
			currentChild = root.getNextChild(currentChild);
			paintNode(gc, currentChild, currentChild.getRGBColor());
			paintLine(gc, root, currentChild);
			paint(gc, currentChild,1);
		}
		
	}
	
	
/* ******************************************************** */
	public void paintMainNode(GC gc,Node root){
/* ******************************************************** */
		int x = (int)(root.getStart()*ratioX)+1;
		int y = (int)(root.getY()*ratioY) + (2*offsetY)/3;
		int width = (int)(root.getSize()*ratioX);
		int height = (int)(root.getNodeHeight()*ratioY)+ (offsetY)/3;
		Rectangle rect = new Rectangle(x,y,width ,height);
		
		gc.setLineWidth(4); 
		gc.drawRectangle(rect);
		Color c = root.getRGBColor();
		gc.setBackground(c);
		gc.fillRectangle(rect);
		
		int[] coord = {x,y};
		coordinates.put(root.getId(), coord);
		
		gc.setFont( new Font(getDisplay(),"Arial", 14, SWT.BOLD ) );
		Point pt = gc.stringExtent(this.name);
		gc.drawString(this.name, rect.width/2 - pt.x/2 + rect.x, rect.y + rect.height/2 - pt.y/2);
	}
	
	
/* ******************************************************** */
	public void paintNode(GC gc,Node root, Color c){
/* ******************************************************** */
		gc.setForeground(new Color(null, 0,0,0));
		gc.setLineWidth(2);
		int x = (int)(root.getStart()*ratioX)+1;
		int y = (int)(root.getY()*ratioY) + offsetY;
		int width = (int)(root.getSize()*ratioX);
		int height = (int)(root.getNodeHeight()*ratioY);
		
		gc.setBackground(c);
		drawPolygon(gc,root.getKind(),x,y,width,height);
		
		int[] coord = {x,y};
		coordinates.put(root.getId(), coord);
	}
	
/* ******************************************************** */
	public void paintLine(GC gc,Node parent, Node child){
/* ******************************************************** */
		int x1, y1, x2, y2;
		x1=(int)( parent.getStart()*ratioX + (parent.getSize()*ratioX) / 2);
		y1= (int)(parent.getY()*ratioY  +(parent.getNodeHeight()*ratioY)+1) + offsetY;
		x2 = (int)(child.getStart()*ratioX + (child.getSize()*ratioX)/2);
		y2 = (int)(child.getY()*ratioY-1) + offsetY;
		
		gc.setForeground(new Color(null, 216,214,206));
		gc.setLineWidth(1);
		gc.setAntialias(SWT.ON);
		gc.drawLine(x1, y1, x2, y2);
	}
	
/* ******************************************************** */
	public void drawPolygon(GC gc,String kind,int x,int y,int width,int height){
/* ******************************************************** */
		if(kind=="if")
			drawTrapeze(gc,x,y,width,height);
		else if(kind=="else")
			drawInvaTrapeze(gc,x,y,width,height);
		else if(kind=="for")
			gc.fillOval(x, y, width, height);
		else if(kind=="while")
			gc.fillOval(x, y, width, height);
		else if(kind=="switch")
			drawHexagon(gc,x,y,width,height);
		else if(kind=="case")
			drawTrapeze(gc,x,y,width,height);
	}
	
/* ******************************************************** */
	public void drawTrapeze(GC gc,int x,int y,int width,int height){
/* ******************************************************** */
		int[] pointArray={x, y+height, x+width, y+height, x+width/4*3, y, x+width/4, y};
		//gc.drawPolygon(pointArray);
		gc.fillPolygon(pointArray);
		
	}
	
/* ******************************************************** */
	public void drawInvaTrapeze(GC gc,int x,int y,int width,int height){
/* ******************************************************** */
		int[] pointArray={ x+width/4, y+height, x+width/4*3, y+height, x+width, y, x, y};
		//gc.drawPolygon(pointArray);
		gc.fillPolygon(pointArray);
		
	}
	
/* ******************************************************** */
	public void drawHexagon(GC gc,int x,int y,int width,int height){
/* ******************************************************** */
		int[] pointArray={x,y+height/2,x+width/3,y+height,x+width*2/3,y+height,x+width,y+height/2,x+width*2/3,y,x+width/3,y};
	//	gc.drawPolygon(pointArray);
		gc.fillPolygon(pointArray);
		
	}
	
/* ******************************************************** */
	public void clearCanvas(GC gc){
/* ******************************************************** */
		gc.setBackground(new Color(getDisplay(), 255,255,255));
		gc.fillRectangle(getClientArea());
		gc.setBackground(new Color(getDisplay(), 0,0,0));
	}
	
	
/* ******************************************************** */	
	public void drawLegend(GC gc,int ratio){
/* ******************************************************** */
		
		int height = 35;//(int)(root.getNodeHeight()*ratioY+2*ratioY);
		gc.setTextAntialias(SWT.ON);
		gc.setAntialias(SWT.ON);
		if(clientWidth>400)
			gc.setFont( new Font(getDisplay(),"Arial", 10, SWT.BOLD ) );
		else if(clientWidth>250)
			gc.setFont( new Font(getDisplay(),"Arial",7, SWT.BOLD ) );
		else if(clientWidth>150)
			gc.setFont( new Font(getDisplay(),"Arial",5, SWT.BOLD ) );
		else
			gc.setFont( new Font(getDisplay(),"Arial",3, SWT.BOLD ) );
		
		gc.setForeground(new Color(getDisplay(), 255,255, 255));
		
		/**********   IF  ***********/
		//Rectangle rect1 = new Rectangle(1, 2, getClientArea().width / 6  - 2,height);
		gc.setBackground(new Color(getDisplay(), 255, 224, 54));
		//gc.fillRectangle(rect1);
		drawTrapeze(gc,1, 2, clientWidth / 6  - 2,height);
		//FontMetrics fm = gc.getFontMetrics();
		Point pt = gc.stringExtent("IF");
		gc.drawString("IF",  clientWidth / 12 - pt.x/2 + 1, 2 + height/2 - pt.y/2,true);
		//gc.drawString("IF",1 + getClientArea().width / 13, height/3,true);
		
		
		/**********   ELSE  ***********/
		//Rectangle rect2 = new Rectangle(1+(getClientArea().width / 6), 2, getClientArea().width / 6 - 2 ,height);
		gc.setBackground(new Color(getDisplay(), 232, 140, 41));
		//gc.fillRectangle(rect2);
		drawInvaTrapeze(gc,1+(clientWidth / 6), 2, getClientArea().width / 6 - 2 ,height);
		//fm = gc.getFontMetrics();
		pt = gc.stringExtent("ELSE");
		gc.drawString("ELSE",  clientWidth / 12 - pt.x/2 + 1+(clientWidth / 6), 2 + height/2 - pt.y/2,true);
		//gc.drawString("ELSE",1+(getClientArea().width / 6) + getClientArea().width / 15, height/3,true);
		
		/**********   FOR  ***********/
		//Rectangle rect3 = new Rectangle(1+(getClientArea().width / 6)*2, 2, getClientArea().width / 6  - 2,height);
		gc.setBackground(new Color(getDisplay(), 100, 200 ,200));
		//gc.fillRectangle(rect3);
		gc.fillOval(1+(clientWidth / 6)*2, 2, clientWidth / 6  - 2,height);
		//fm = gc.getFontMetrics();
		pt = gc.stringExtent("FOR");
		gc.drawString("FOR",  getClientArea().width / 12 - pt.x/2 + 1+(clientWidth / 6)*2, 2 + height/2 - pt.y/2,true);
		//gc.drawString("FOR",1+(getClientArea().width / 6)*2 + getClientArea().width / 15, height/3,true);
		
		/**********   WHILE  ***********/
		//Rectangle rect4 = new Rectangle(1+(getClientArea().width / 6)*3, 2, getClientArea().width / 6  - 2,height);
		gc.setBackground(new Color(getDisplay(), 53, 203, 120));
		//gc.fillRectangle(rect4);
		gc.fillOval(1+(clientWidth / 6)*3, 2, clientWidth / 6  - 2,height);
		//fm = gc.getFontMetrics();
		pt = gc.stringExtent("WHILE");
		gc.drawString("WHILE",  clientWidth / 12 - pt.x/2 + 1+(clientWidth / 6)*3, 2 + height/2 - pt.y/2,true);
		//gc.drawString("WHILE",1+(getClientArea().width / 6)*3 + getClientArea().width / 17, height/3,true);
		
		/**********   SWITCH  ***********/
	//	Rectangle rect5 = new Rectangle(1+(getClientArea().width / 6)*4, 2, getClientArea().width / 6  - 2,height);
		gc.setBackground(new Color(getDisplay(), 209, 61, 61));
	//	gc.fillRectangle(rect5);
		drawHexagon(gc,1+(clientWidth / 6)*4, 2, clientWidth / 6  - 2,height);
		//fm = gc.getFontMetrics();
		pt = gc.stringExtent("SWITCH");
		gc.drawString("SWITCH",  clientWidth / 12 - pt.x/2 + 1+(clientWidth / 6)*4, 2 + height/2 - pt.y/2,true);
		//gc.drawString("SWITCH",1+(getClientArea().width / 6)*4 + getClientArea().width / 17, height/3,true);
		
		/**********   CASE  ***********/
		//Rectangle rect6 = new Rectangle(1+(getClientArea().width / 6)*5, 2, getClientArea().width / 6  - 2,height);
		gc.setBackground(new Color(getDisplay(), 93, 90, 161));
		//gc.fillRectangle(rect6);
		drawTrapeze(gc, 1+(clientWidth/ 6)*5, 2, clientWidth / 6  - 2,height);
		//fm = gc.getFontMetrics();
		pt = gc.stringExtent("CASE");
		gc.drawString("CASE",  clientWidth / 12 - pt.x/2 + 1+(clientWidth / 6)*5, 2 + height/2 - pt.y/2,true);
		//gc.drawString("CASE",1+(getClientArea().width / 6)*5 + getClientArea().width / 16, height/3,true);
		
	}
	
/* ******************************************************** */	
	public void writeContentToFile(String filename) {
/* ******************************************************** */	
        GC gc = new GC(thisCanvas);
        Image image = new Image(this.getDisplay(), thisCanvas.getSize().x,
                thisCanvas.getSize().y);
      //zeichneDiagramm(gc, this.getDisplay());
        gc.copyArea(image, 0, 0);
        ImageLoader loader = new ImageLoader();
        loader.data = new ImageData[] { image.getImageData() };
        
        FileDialog fd = new FileDialog(getShell(), SWT.SAVE);
        fd.setText("Save");
        fd.setFilterPath("C:/");
        String[] filterExt = { "*.jpg", "*.jpeg", "*.*" };
        fd.setFilterExtensions(filterExt);
        fd.setFileName(filename);
        //fd.setOverwrite(true);
        String selected = fd.open();
        filename = selected; 
        loader.save(filename, SWT.IMAGE_JPEG);
        gc.dispose();
    }

/* ******************************************************** */
public boolean setTooltip(int x, int y, int idx){
/* ******************************************************** */
		
		for(Integer key : coordinates.keySet()){
			int[] coord = coordinates.get(key);
			final Node node = aTree.getNode(aTree.getRoot(), key);
			Rectangle rect = new Rectangle(coord[0],coord[1],(int)(node.getSize()*ratioX),(int)(node.getNodeHeight()*ratioY));
			
			if(rect.contains(x, y))
			{
				if(idx==0)
					tip.clear();
				ToolTip tmp =  new DefaultToolTip(thisCanvas,SWT.NONE, true) {
                    @Override
                    public String getText( Event event )
                    {
                    	String code = node.getCode();
                    	code = code.replaceAll("\t", " ");
                    	return code;
                    }
                    
                };
				tip.add(idx,tmp);
                tip.get(idx).show(new Point(coord[0]-((int)(node.getSize()*ratioX)/2 - 10),coord[1]+((int)(node.getNodeHeight()*ratioY))+10 ));
                return true;
			}
		}
		return false;
	}

}
